# -*- perl -*-

#    Copyright (C) 1995  Ken MacLeod
#    This file is part of DTD Fragments.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

require DTD::CptrTerms;

package SROFF;

%main::map = (%main::map,
# admon.class
	      "CAUTION", "DBFormal",
	      "IMPORTANT", "DBFormal",
	      "NOTE", "DBFormal",
	      "TIP", "DBFormal",
	      "WARNING", "DBFormal",

# appendix.class
	      "APPENDIX", "Section",

# article.class
	      "ARTICLE", "Holder",

# base.char.class
				# Anchor in genobj.class

# book.class
	      "BOOK", "Holder",

# chapter.class
	      "CHAPTER", "Section",

# compound.class
				# MsgSet
				# Procedure
				# SideBar

# cptr.char.class in DTD::CptrTerms

# descobj.class
	      "ABSTRACT", "Abstract",
	      "AUTHORBLURB", "Blockquote",
	      "EPIGRAPH", "Holder",

# docinfo.char.class
	      "AUTHOR", "Holder",
	      "CORPAUTHOR", "Holder",
				# ModeSpec
				# OtherCredit
	      "PRODUCTNAME", "InlineBold",
				# ProductNumber
	      "REVHISTORY", "WarnIgnore",

# formal.class
				# Equation
	      "EXAMPLE", "DBFigure",
	      "FIGURE", "DBFigure",
	      "TABLE", "DBFigure",

# genobj.class
	      "ANCHOR", "WarnIgnore",
	      "BRIDGEHEAD", "DBBridgeHead",
				# Comment in other.char.class
				# Highlights

# index.class
				# Index
				# SetIndex

# informal.class
	      "BLOCKQUOTE", "Blockquote",
				# InformalEquation
				# InformalExample
				# InformalTable
	      "GRAPHIC", "WarnIgnore",

# inlineobj.char.class
				# InlineEquation
				# InlineGraphic

# linespecific.class
	      "LITERALLAYOUT", "Screen",
	      "PROGRAMLISTING", "Screen",
	      "SCREEN", "Screen",
				# ScreenShot

# link.char.class
	      "LINK", "Inline",
				# OLink
	      "ULINK", "Inline",

# list.class
				# GlossList
	      "ITEMIZEDLIST", "ItemizedList",
	      "ORDEREDLIST", "DBOrderedList",
				# Segmentedlist
				# SimpleList
	      "VARIABLELIST", "VariableList",

# nav.class
	      "TOC", "WarnIgnore",
				# LoT
				# Index in index.class
				# Glossary
				# Bibliography

# ndxterm.class
	      "INDEXTERM", "WarnIgnore",

# notation.class

# other.char.class
	      "COMMENT", "WarnIgnore",
				# SubScript
				# SuperScript

# para.class
	      "FORMALPARA", "DBFormal",
	      "PARA", "Paragraph",
	      "SIMPARA", "Paragraph",

# refentry.class
				# RefEntry

# synop.class
	      "SYNOPSIS", "Abstract",
				# CmdSynopsis
				# FuncSynopsis

# word.char.class in DTD::CptrTerms

# xref.char.class
	      "FOOTNOTEREF", "WarnIgnore",
	      "XREF", "DBXRef",

# ------------

# Temporary hacks
	      "ARTHEADER", "DBDocInfo",
	      "ORGDIV", "Holder",
	      "ADDRESS", "Holder",
	      "EMAIL", "Holder",
	      "ARTPAGENUMS", "WarnIgnore",
	      "EDITION", "WarnIgnore",
	      "COLSPEC", "WarnIgnore",
	      "SPANSPEC", "WarnIgnore",
# Structure
	      "PREFACE", "Section",
	      "DOCINFO", "DBDocInfo",
	      "BOOKINFO", "DBDocInfo",
	      "SECT1", "Section",
	      "SECT2", "Section",
	      "SECT3", "Section",
	      "SECT4", "Section",
	      "SECT5", "Section",
	      "TITLEABBREV", "WarnIgnore",
	      "HIGHLIGHTS", "Holder",


# BOOKINFO
	      "BOOKBIBLIO", "Holder",
	      "RELEASEINFO", "WarnIgnore",
	      "COPYRIGHT", "Holder",
	      "YEAR", "Holder",
	      "HOLDER", "Holder",
	      "LEGALNOTICE", "LegalNotice",
	      "PUBDATE", "Holder",

# DOCINFO
	      "AUTHORGROUP", "WarnIgnore",
	      "FIRSTNAME", "FirstName",
	      "SURNAME", "SurName",
	      "AFFILIATION", "Holder",
	      "ORGNAME", "Holder",
	      "REVISION", "WarnIgnore",
	      "REVNUMBER", "WarnIgnore",
	      "DATE", "WarnIgnore",

# TOC
	      "TOCCHAP", "WarnIgnore",
	      "TOCLEVEL1", "WarnIgnore",
	      "TOCLEVEL2", "WarnIgnore",
	      "TOCENTRY", "WarnIgnore",

	      "PRIMARY", "WarnIgnore",
	      "SECONDARY", "WarnIgnore",
	      "TERTIARY", "WarnIgnore",

#
	      "LISTITEM", "ListItem",
	      "VARLISTENTRY", "Holder",
	      "TERM", "Term",

# Linking
	      "FOOTNOTE", "Footnote",
	      "SEE", "WarnIgnore",

# Tables
	      "TGROUP", "WarnIgnore",
	      "THEAD", "WarnIgnore",
	      "TBODY", "WarnIgnore",
	      "ROW", "WarnIgnore",
	      "ENTRY", "WarnIgnore",

#
	      "TITLE", "Title",
	      "SUBTITLE", "Title",

	      "LINEANNOTATION", "LineAnnotation",


	     );

#
# Initialize the context for this DTD
#

sub InitDTD {
    my ($self, $gi) = @_;

    $self->{refTypeMap} = {CHAPTER, "Chapter",
			   APPENDIX, "Appendix",
			   PREFACE, "Preface",
			   SECT1, "Section",
			   SECT2, "Section",
			   SECT3, "Section",
			   SECT4, "Section",
			   SECT5, "Section",
			   EXAMPLE, "Example",
			   FIGURE, "Figure",
			   TABLE, "Table"
			   };
}

#########################################################################
#
# DocBook specialized elements
#

sub enDBDocInfo {
    my ($self, $gi) = @_;

    $self->map ("TITLE", "StructTitle",
	      "SUBTITLE", "StructSubTitle",
	      "DATE", "StructDate",
	      "AUTHOR", "DBAuthor",
	      "CORPAUTHOR", "DBCorpAuthor",
	      "YEAR", "DBCopyYear",
	      "HOLDER", "DBCopyHolder",
	      "COPYRIGHT", "DBCopyright",
	      );
    unshift (@{$self->{docRef}}, new Referable::Doc);
}
sub lvDBDocInfo {
    my ($self, $gi) = @_;

    StructTitlePage $self;
    shift (@{$self->{docRef}});
    $self->unmap ("TITLE", "DATE", "AUTHOR", "CORPAUTHOR", "YEAR",
		      "HOLDER",	"COPYRIGHT");
}

sub enDBCorpAuthor {
    my ($self, $gi) = @_;

    unshift (@{$self->{docRef}[0]{authors}}, {});
    enStructAuthor $self ($gi);
}

sub lvDBCorpAuthor {
    my ($self, $gi) = @_;

    lvStructAuthor $self ($gi);
}

sub enDBAuthor {
    my ($self, @rest) = @_;

    unshift (@{$self->{docRef}[0]{authors}}, {});
    $self->{firstName} = $self->{surName} = undef;
    $self->enStructAuthor(@rest);
}

sub lvDBAuthor {
    my ($self, @rest) = @_;

    $self->{data} = "";
    if (defined $self->{firstName}) {
	$self->{data} .= "$self->{firstName}";
	$self->{data} .= " "
	    if (defined $self->{surName});
    }
    $self->{data} .= "$self->{surName}"
	if (defined $self->{surName});
    $self->lvStructAuthor(@rest);
}

sub enFirstName {
    my ($self, $gi) = @_;

}
sub lvFirstName {
    my ($self, $gi) = @_;

    $self->{firstName} = $self->{data};
    $self->{data} = "";
}

sub enSurName {
    my ($self, $gi) = @_;

}
sub lvSurName {
    my ($self, $gi) = @_;

    $self->{surName} = $self->{data};
    $self->{data} = "";
}

sub enAffiliation {
    my ($self, $gi) = @_;

}
sub lvAffiliation {
    my ($self, $gi) = @_;

}

sub enOrgName {
    my ($self, $gi) = @_;

}
sub lvOrgName {
    my ($self, $gi) = @_;

    $self->{orgName} = $self->{data};
    $self->{data} = "";
}

sub enDBCopyright {
    my ($self, $gi) = @_;

    $self->{copyYear} = $self->{copyHolder} = ();
}
sub lvDBCopyright {
    my ($self, $gi) = @_;

    $self->{docRef}[0]{copyright} = join (", ", $self->{copyYear}) . " "
	. join (", ", $self->{copyHolder});
    if (ref ($self) =~ /ASCII/) {
	$self->{docRef}[0]{copyright} = "\1"
	    . $self->fill("Copyright (C) " . $self->{docRef}[0]{copyright})
	    . "\1";
    }
    $self->{data} = "";
}

sub enDBCopyYear {
    my ($self, $gi) = @_;

}
sub lvDBCopyYear {
    my ($self, $gi) = @_;

    unshift (@{$self->{copyYear}}, $self->{data});
    $self->{data} = "";
}

sub enDBCopyHolder {
    my ($self, $gi) = @_;

}
sub lvDBCopyHolder {
    my ($self, $gi) = @_;

    unshift (@{$self->{copyHolder}}, $self->{data});
    $self->{data} = "";
}

sub enLegalNotice {
    my ($self, $gi) = @_;

}
sub lvLegalNotice {
    my ($self, $gi) = @_;

    $self->{docRef}[0]{otherInfo} .= $self->{data};
    $self->{data} = "";
}

sub enDBFigure {
    my ($self, $gi) = @_;

    $self->map ("TITLE", "FigureTitle",
	      );
    $self->enFigure($gi);
}
sub lvDBFigure {
    my ($self, $gi) = @_;

    $self->lvFigure($gi);
    $self->unmap ("TITLE");
}

sub enDBXRef {
    my ($self, $gi) = @_;

}
sub lvDBXRef {
    my ($self, $gi) = @_;

    $self->{data} = "\035ref $::attribute_value{$::level,'LINKEND'}\035";
}

#
# Formal is an element that may have a title, use BridgeHead to format it.
#
sub enDBFormal {
    my ($self, @rest) = @_;
    $self->map ("TITLE", "DBBridgeHead");
}
sub lvDBFormal {
    my ($self, @rest) = @_;
    $self->unmap ("TITLE");
}
    

#
# BridgeHead is a heading that separates/bridges two sections...
#
# I think.
#
sub enDBBridgeHead {
    my ($self, $gi) = @_;

}

#lvDBBridgeHead is output-format specific

#
# DBOrderedList has an attribute that allows numbering to be continued
# from the previous list
#

sub enDBOrderedList {
    my ($self, @rest) = @_;

    $self->enOrderedList (@rest);
    if ($::attribute_value{$::level, 'CONTINUATION'}
	=~ /^CONTINUES$/i) {
	$self->{itemnum}[0] =
	    $self->{lastItemNum}[$#{$self->{itemnum}}];
    }
}

sub lvDBOrderedList {
    my ($self, @rest) = @_;

    $self->{lastItemNum}[$#{$self->{itemnum}}]
	= $self->{itemnum}[0];
    $self->lvOrderedList(@rest);
}

#########################################################################
#
# HTML Specific DocBook
#

package SROFF::HTML;

sub lvDBBridgeHead {
    my ($self, $gi) = @_;

    $self->{data} = "\n<H4>" . $self->{data} . "</H4>\n";
}

#########################################################################
#
# ASCII Specific DocBook
#

package SROFF::ASCII;

sub lvDBBridgeHead {
    my ($self, $gi) = @_;

    $self->{data} = "\01" . $self->{data} . "\01";
}

#########################################################################
#
# TROFF Specific DocBook
#

package SROFF::TROFF;

sub lvDBBridgeHead {
    my ($self, $gi) = @_;

    $self->{data} = "\0.LP\n\\fB" . $self->{data}
	. "\\f$self->{font}[0]\0.br\n";
}

1;

__END__
#
# The following are as yet unimplemented DOCBOOK elements.  If used,
# they will cause an error, hinting that maybe you should implement
# them :-)
#
ackno
address
appendix
arg
artheader
artpagenums
authorinitials
beginpage
bibliodiv
biblioentry
bibliomisc
city
collab
collabname
colspec
confdates
confgroup
confnum
confsponsor
conftitle
contractnum
contractsponsor
contrib
corpname
country
data
edition
editor
email
entry
entrytbl
fax
funcdef
funcparams
funcsynopsisinfo
glossdef
glossdiv
glossentry
glosssee
glossseealso
glossterm
graphic
group
honorific
indexdiv
indexentry
indexterm
invpartnumber
isbn
issn
issuenum
jobtitle
journalinfo
lineage
lotentry
manvolnum
member
msg
msgaud
msgentry
msgexplan
msginfo
msglevel
msgmain
msgorig
msgrel
msgsub
olink
orgdiv
otheraddr
othername
pagenums
paragraphs
paramdef
part
partintro
phone
pob
postcode
primaryie
printhistory
programlisting
publisher
publishername
pubsnumber
refclass
refdescriptor
refentrytitle
reference
refmeta
refmiscinfo
refname
refnamediv
refpurpose
refsect1
refsect2
refsect3
refsynopsisdiv
revremark
row
screeninfo
secondaryie
see
seealso
seealsoie
seeie
seg
seglistitem
segmentedlist
segtitle
seriesinfo
seriesvolnums
set
setinfo
shortaffil
simplelist
spanspec
state
step
street
substeps
synopfragment
synopfragmentref
synopsis
table
tbody
tertiary
tertiaryie
tfoot
tgroup
thead
tocback
tocfront
toclevel2
tocpart
varargs
void
volumenum
xref
