# -*- perl -*-

#    Copyright (C) 1995  Ken MacLeod
#    This file is part of DTD Fragments.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

package SGML;

#
# `new' is used by formatters and styles to initialize the context
#

sub new {
    my $self;

    $self = {};

    bless $self;
}

#
# `Initialize's only current use is to set the global
# `backslash_in_data' according to the current output type.  This
# should be part of the context.
#

sub Initialize {
}

#
# `PostProcess' occurs after the entire input file has been read and
# formatted.  The formatters now resolve references, remove
# unnecessary constructs (making formatting easier), and hopefully
# simplify the output
#
sub PostProcess {
    my ($self, @rest) = @_;

    print $self->{data};
}

#
# `map' and `unmap' allow for an element mapping to be overridden
# within an element context.  For example, DocBook uses the `Title'
# element for section titles and figure titles, but the general
# formatter handles them differently; the DocBook processor has a
# `Figure' method that changes the mapping for `Title'.
#
sub map {
    my ($self, %newMapping) = @_;

    while (($key, $value) = each %newMapping) {
	unshift (@{$self->{mapStack}->{$key}}, $main::map{$key});
	$main::map{$key} = $value;
#	eval ("unshift (\@${key}Inheritance, \$inherits{$key}); \$inherits{$key} = '$value'");
    }
}

sub unmap {
    my ($self, @keys) = @_;

    foreach $key (@keys) {
	$main::map{$key} = shift (@{$self->{mapStack}->{$key}});
#	eval ("\$inherits{$key} = shift (\@${key}Inheritance)");
    }
}

#
# `en' is a safety net.  It is called as a side-effect of the mapping
# lookup when there is no mapping for an element.
# XXX In the future it may call `WarnIgnore', although, `en' means the
# programmer forgot the element and `WarnIgnore' means the programmer
# chose to not implement the element (yet).
#
sub en {
    my ($self, $gi) = @_;
    
    die "undefined element at line $::lineno: $gi\n";
}

#
# `addReferable' adds a referable it to our context.
#
sub addReferable {
    my ($self, $referable) = @_;

    unshift (@{$self->{referables}}, $referable);
    if (defined $referable->{id}) {
	$self->{referablesByID}{"ref $referable->{id}"} = $referable;
    }

    return $referable;
}

#
# Holder is a generic element that merely contains, and outputs, other
# elements.  Generally in the DTDs these elements mark off their
# sub-elements but do not produce visible formatting.
#

sub enHolder {
}

sub lvHolder {
}

#
# Drop is a generic element that contains, but does not output, other
# elements.  Generally in the DTDs this is information that would not
# be needed in the output.
#

sub enDrop {
}
sub lvDrop {
    my ($self, @rest) = @_;
    $self->{data} = "";
}

#
# WarnIgnore is like Drop except that it warns of not handling the
# content.  Generally these are elements that the programmer has not
# yet handled in developing the output processors.
#
sub enWarnIgnore {
    my ($self, $gi) = @_;

    if (!defined $self->{warnIgnored}{$gi}) {
	$self->{warnIgnored}{$gi} = 1;
	warn "$gi: not handled by this version\n";
    }
}
sub lvWarnIgnore {
    my ($self, @rest) = @_;
    $self->{data} = "";
}

1;
