# -*- perl -*-

#    Copyright (C) 1995  Ken MacLeod
#    This file is part of DTD Fragments.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

require SGML;
require Referable;
require DocReferable;

package SROFF;

@ISA = (SGML);

#
# SLOTS
#     filling -- Stack
#        `filling[0]' true if the contents of this element should be
#        filled/wrapped.
#
#     quoting -- Stack
#        `quoting[0]' true if we are already inside of quotes.
#
#     data -- String
#        Contains the (being) formatted content of this element, from
#        `dataStack[0]'
#
#     dataStack -- Stack
#        Contains the yet to be formatted content.  `dataStack[0]'
#        contains all the formatted content at ``this'' element level,
#        i.e. `data' is concatenated to `dataStack[0]' after each
#        element.
#
#     sectionNum -- Stack
#        `sectionNum[0]' contains the section number of the next
#        subsection.  `sectionNum[1]' contains the section number of
#        ``this'' section.
#
#     docRef -- Stack of Referable::Doc
#        `docRef[0]' contains overall document information
#
#

sub new {
    my $self;

    $self = new SGML;

    $self->{filling} = [0];
    $self->{quoting} = [1];

    $self->{data} = "";
    $self->{dataStack} = [""];

    $self->{sectionNum} = [0];

    $self->{docRef} = [];

    bless $self;
}

#sub Initialize {}
#sub PostProcess {}

#
# fmtReference retreives a formatted copy of a Referable, if it's
# available
#

sub fmtReference {
    my ($self, $refId) = @_;

    if (ref ($self->{referablesByID}{$refId}) =~ /Referable/) {
	return $self->{referablesByID}{$refId}->fmtReference ($self);
    } else {
	return "[Unknown Reference]";
    }
}

1;
