# -*- perl -*-

#    Copyright (C) 1995  Ken MacLeod
#    This file is part of DTD Fragments.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

require SROFF;
require SROFF::ASCII::CharEntities;
require SROFF::ASCII::Inline;
require SROFF::ASCII::Lists;
require SROFF::ASCII::Paragraph;
require SROFF::ASCII::Structure;

package SROFF::ASCII;

@ISA = (SROFF);

$fillOn = "\036";
$fillOff = "\037";

sub new {
    my $self;

    $self = new SROFF;

    $self->{parHang}[0] = 0;
    $self->{rindent}[0] = 0;
    $self->{indent}[0] = 0;

    bless $self;
}

sub Initialize {
    my ($self, @rest) = @_[0];

    $::backslash_in_data = "\\";

    $self->SROFF::Initialize (@rest);
}

sub PostProcess {
    my ($self, @rest) = @_;

    $self->{data} = $self->{dataStack}[0];
    $self->{data} =~ s/\035([^\035]*)\035/$self->fmtReference($1)/eg;
    $self->{data} =~ s/\036([0-9]+);([0-9]+);([0-9]+);([^\037]*)\037/$self->fmt($1,$2,$3,$4)/eg;
    # By including whitespaces before newlines-wanted we
    # get whitespace at end of lines (maybe bad?) but also catch
    # newlines-wanted separated by filling/formatting
    $self->{data} =~ s/((\s*[\0\1\2\3\4\5\6\7])+)/&newlines($1)/eg;

    $self->SROFF::PostProcess (@rest);
}

sub newlines {
    local ($chars) = $_[0];
    local ($max, @chars);
    # clean whitespace because the caller's search string includes them
    $chars =~ s/\s//g;
    @chars = unpack ("C*", $chars);
    $max = 0;
    foreach $char (@chars) {
	$max = $char + 0
	    if (($char + 0) > $max);
    }
    "\n" x ($max + 1);
}


###
### Output formatting helpers
###

#
# undertitle underscores titles in `ascii' output
#
sub undertitle {
    local ($self, $repchar, $title) = @_;
    $str = $repchar x length ($title);
    return $str;
}

#
# center centers text in `ascii' output
#

sub center {
    my ($self, $str) = @_;
    $str = " " x (38 - length ($str) / 2) . $str;
    return $str;
}

#
# fill causes text within to be filled in `ascii' output
#

sub fill {
    my ($self, $str) = @_;
    "$fillOn${$self->{indent}}[0];${$self->{parHang}}[0];"
	. "${$self->{rindent}}[0];$str$fillOff";
}

#
# nofill causes text within to not be filled `ascii' output
#

sub nofill {
    my ($self, $str) = @_;
    "$fillOff$str$fillOn${$self->{indent}}[0];${$self->{parHang}}[0];"
	. "${$self->{rindent}}[0];";
}

#
# fmt re-fills paragraphs (like fmt(1)) in `ascii' output
# it is called from the top-level post-processor
#

sub fmt {
    local ($self, $indent, $hang, $rindent, $str) = @_;
    local ($out, $lineWidth, $pos);

    $str =~ s/\s+/ /g;
    $str =~ s/^\s//;
    $out = "";
    $lineWidth = 76 - $indent - $rindent;
    while (length ($str) > $lineWidth) {
	$pos = rindex ($str, " ", $lineWidth);
	if ($pos == -1) {
	    $pos = $lineWidth;
	}
	$left = " " x $indent . substr ($str, 0, $pos) . "\n";
	$out .= $left;
	$str = substr ($str, $pos + 1);
    }
    $out .= " " x $indent . $str;
    substr ($out, $hang);
}

1;
