# -*- perl -*-

#    Copyright (C) 1995  Ken MacLeod
#    This file is part of DTD Fragments.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

package SROFF;

#
# Paragraph is probably the most basic block element of most DTDs.
# `$firstPara' processing in most output formats is important
# because of the way hanging tags and indents are treated
#

sub enParagraph {
    my ($self, @rest) = @_;
    unshift (@{$self->{firstPara}}, 0);
    unshift (@{$self->{filling}}, 1);
}

sub lvParagraph {
    my ($self, @rest) = @_;
    shift (@{$self->{filling}});
    shift (@{$self->{firstPara}});
}

#
# Screen is used for ``verbatim'', ``literal'', ``program listing'',
# etc. type formatting.  Line-ends are kept in the output format.
#

sub enScreen {
    my ($self, @rest) = @_;
    unshift (@{$self->{quoting}}, 0);
    unshift (@{$self->{filling}}, 0);
}

sub lvScreen {
    my ($self, @rest) = @_;
    shift (@{$self->{filling}});
    shift (@{$self->{quoting}});
}

#
# LineAnnotation (DocBook specific???)
# Often in program listings, line annotations are used to make a margin note.
#

sub enLineAnnotation {
    my ($self, @rest) = @_;
    unshift (@{$self->{quoting}}, 1);
    $self->enInlineItalic(@rest);
}

sub lvLineAnnotation {
    my ($self, @rest) = @_;
    $self->lvInlineItalic(@rest);
    shift (@{$self->{quoting}});
}

#
# BlockQuote is a group of block elements that are set off from their
# surroundings, typically by indentation.
#
sub enBlockQuote {
    my ($self, @rest) = @_;
    unshift (@{$self->{filling}}, 0);
}

sub lvBlockQuote {
    my ($self, @rest) = @_;
    shift (@{$self->{filling}});
}

#
# Figure contains figures and their captions
#
sub enFigure {
    my ($self, $gi) = @_;
    $self->{figureNum} ++;
    $self->{figure}
    = $self->addReferable (new Referable (index => $self->{figureNum},
					  type => $self->{refTypeMap}{$gi},
					  id => $::attribute_value{$::level,'ID'}));
}

sub lvFigure {
    my ($self, @rest) = @_;
    # This makes the assumption that the figure caption follows the
    # figure in the output stream.  Must be overridden if this is not
    # the case
    $self->{data} = $self->{data} . $self->{figureTitle};
}

#
# FigureTitles appear within ``Figure'' elements.
#
sub enFigureTitle {
}
sub lvFigureTitle {
    my ($self, @rest) = @_;
    # As in `Figure', this makes an assumption that may need to be
    # overridden
    $self->{figureTitle} = $self->{data};
    $self->{data} = "";
}

#
# Footnotes
#
sub enFootnote {
}
sub lvFootnote {
}

1;
