# -*- perl -*-

#    Copyright (C) 1995  Ken MacLeod
#    This file is part of DTD Fragments.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#
# The structure module captures front matter and sectioning
# information.
#
# <!ELEMENT FrontMatter - -
#     (Title, SubTitle?, Authors*, Abstract?, OtherInfo?, Copyright?)>
# <!ELEMENT Authors - -
#     ((Author | CorporateAuthor | Editor), AuthorAffiliation?)>
#
# Sections are recursive
# <!ELEMENT Section - -
#     (Title?, BodyElements*, Section*)>
#
# These are ``inline'' or single line elements
# <!ELEMENT (Title, SubTitle, Author, CorporateAuthor, AuthorAffiliation) - -
#     (#PCDATA | InlineElements)>
#
# These are ``block'', ``body'', or multiple line elements
# <!ELEMENT (Abstract, OtherInfo, Copyright) - -
#     (BodyElements)>
#

use SROFF;

package SROFF;

#
#
#

sub enStructTitle {
}

sub lvStructTitle {
    my ($self, @rest) = @_;
    $self->{docRef}[0]{title} = $self->{data};
    $self->{data} = "";
}

#
#
#

sub enStructSubTitle {
}

sub lvStructSubTitle {
    my ($self, @rest) = @_;
    $self->{docRef}[0]{subTitle} = $self->{data};
    $self->{data} = "";
}

#
#
#

sub enStructDate {
}

sub lvStructDate {
    my ($self, @rest) = @_;
    $self->{docRef}[0]{date} = $self->{data};
    $self->{data} = "";
}

#
# The `{docRef}[0]{authors}' array must be created and insert author
# structures by a DTD specific element handler.  Some DTDs have only a
# single `Author' element, so  the ``document'' element must create
# it, some have a header element  with a list of authors that must
# create it.
#

sub enStructAuthor {
}
sub lvStructAuthor {
    my ($self, @rest) = @_;
    $self->{docRef}[0]{authors}[0]{author} = $self->{data};
    $self->{data} = "";
}

#
#
#

sub StructTitlePage {
    my ($self, @rest) = @_;

    # by the time SROFF::StructTitlePage is called, `data' includes
    # almost the whole title page

    ##
    ## OtherInfo
    ##
    if (defined ($self->{docRef}[0]{otherInfo})) {
	$self->{data} .= $self->{docRef}[0]{otherInfo};
    }
    if (defined ($self->{docRef}[0]{copyright})) {
	$self->{data} .= $self->{docRef}[0]{copyright};
    }
}

#
# Abstract
#
sub enAbstract {
}

sub lvAbstract {
    my ($self, @rest) = @_;
    $self->{docRef}[0]{abstract} = $self->{data};
    $self->{data} = "";
}

#
# Section contains block elements and other sections.  Multiple levels
# of sections are handled by recursivly calling Section.  (Beware of
# DTDs that allow breaks in section levels, e.g. DocBook's
# ``RenderAs'' and HTML).
#
# The `sectionNum' stack always has an extra sub section on it.
#
sub enSection {
    my ($self, $gi) = @_;
    my ($ii, $sectionNumStr);

    unshift (@{$self->{sectionNum}}, 0);
    $self->{sectionNum}[1] ++;

    for ($ii = $#{$self->{sectionNum}}; $ii > 0; $ii --) {
	$sectionNumStr .= "$self->{sectionNum}[$ii].";
    }
    $sectionNumStr =~ s/\.$//;

    unshift (@{$self->{sectionRefs}},
	     $self->addReferable (new Referable (index => $sectionNumStr,
						 type => $self->{refTypeMap}{$gi},
						 id => $::attribute_value{$::level,'ID'})));
}

sub lvSection {
    my ($self, @rest) = @_;

    shift (@{$self->{sectionRefs}});
    shift (@{$self->{sectionNum}});
}

#
# Titles are section titles and work inside of ``Section'' elements.
#
sub enTitle {
}
sub lvTitle {
}

1;
