#! /usr/local/bin/perl5

#    Copyright (C) 1995  Ken MacLeod
#    This file is part of DTD Fragments.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

$LIBDIR="/usr/local/lib/dtd-fragments";
$LLIBDIR="/usr/local/lib/dtd-fragments";
unshift (@INC, $LLIBDIR, $LIBDIR);

use FileHandle;
require "sgmls.pl";

$prog = $0;

$prog =~ s|.*/||;

sub usage {
    die "usage: $prog -T OUTPUTTYPE -I INPUTTYPE\n";
}

STDERR->autoflush(1);

##############
#
# Argument processing
#

require "getopts.pl";

&Getopts ('T:I:');

$outputType = $opt_T;
$inputType = $opt_I;

if (!defined $inputType) {
    warn "$prog: missing required -I option\n";
    &usage;
} elsif (eval "require $inputType" == undef) {
    die "$prog: Loading input type $inputType: $@\n";
}

if (!defined $outputType) {
    warn "$prog: missing required -T option\n";
    &usage;
} elsif (eval "require $outputType" == undef) {
    die "$prog: Loading output type $outputType: $@\n";
}


##############
#
# Initialization
#

$context = new $outputType;
$context->InitDTD;
$context->Initialize;

##############
#
# Main parse loop
#

&ParseSGMLS($context);		# Will call user* parse routines below

$context->PostProcess();	# and output

exit;

##############
#
# Parse routines
#

#
# Given input data from SGMLS, substitute character entities and
# escaped text.
#
sub user_unescape_data {
    if (ref ($context) =~ /HTML/) {
	$_ =~ s/&/&amp;/g;
	$_ =~ s/</&lt;/g;
    }

    $_[0] =~ s/\\\|(\[[a-zA-Z0-9 ]+\])\\\|/&substEntity($1)/eg;
    $_[0] =~ s/\\([0-7][0-7]?[0-7]?|.)/&esc_data($1)/eg;
}

#
# start_element is called whenever the "(" element-start command is
# found in the SGMLS stream.  Preliminary process is done here, such
# as state setup, item counts, etc.
#
sub user_start_element {
    local($gi) = $_[0];
    unshift (@{$context->{dataStack}}, "");
    $enElement = "en$map{$gi}";
    $context->$enElement ($gi);
}

#
# `end_element' is called whenever the ")" element-end command is
# found in the SGMLS stream.  Final processing is done here, adding a
# segment of formatted output to the current stack bottom.
#
sub user_end_element {
    local($gi) = $_[0];
    $context->{data} = shift(@{$context->{dataStack}});
    $leaveElement = "lv$map{$gi}";
    $context->$leaveElement ($gi);
    ${$context->{dataStack}}[0] .= $context->{data};
}

#
# `data' is called whenever the "-" element-data command is found in
# the SGMLS stream.  Data is always in the context of an open element
# (i.e. between "(" element-start and ")" element-end commands).  We
# just stack it, processing is performed in the element-end function.
#
sub user_data {
    local($data) = $_[0];
    ${$context->{dataStack}}[0] .= $data;
}

#
# Look up a character entity and replace it in the output stream.
#
sub substEntity {
    local ($entity) = $_[0];
    if (defined $entities{$entity}) {
	$entities{$entity};
    } else {
	warn "undefined entity at line $lineno: $entity\n";
	"\\|" . $entity . "\\|";
    }
}

# --- Emacs stuff ---
# Local Variables:
# mode:perl
# End:
