#! /usr/local/bin/perl5
#
#    Copyright (C) 1995  Ken MacLeod
#    This file is part of DTD Fragments.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

$SGMLNORM="sgmlnorm";
$SGMLS="nsgmls";
$LIBDIR="/usr/local/lib/dtd-fragments";
$LLIBDIR="/usr/local/lib/dtd-fragments";
$LIBEXECDIR="/usr/local/lib/dtd-fragments";

unshift (@INC, $LLIBDIR, $LIBDIR);

$prog = $0;

$prog =~ s|.*/||;

sub usage {
    die "usage: $prog [-T OUTPUTTYPE] [-I INPUTTYPE] [-t TYPETAB] [-o OUTPUTFILE] [file]\n";
}

require "getopts.pl";

&Getopts ('T:I:t:o:');

$outputType = $opt_T;
$inputType = $opt_I;
$outputFile = $opt_o;
$typeTab = $opt_t || "typetab";

if ($#ARGV == -1) {
    $document = "-";
} elsif ($#ARGV == 0) {
    $document = $ARGV[0];
    # try `.sgml' and `.sgm' if `$document' isn't found
    if (!-f $document) {
	if (-f "$document.sgml") {
	    $document .= ".sgml";
	} elsif (-f "$document.sgm") {
	    $document .= ".sgm";
	} else {
	    die "$prog: $document: file not found\n";
	}
    }
} else {
    &usage;
}

#
# Find the type table
#
ITER: {
    foreach $prefix (@INC) {
	$realfilename = "$prefix/$typeTab";
	if (-f $realfilename) {
	    $typeTab = $realfilename;
	    last ITER;
	}
    }
    die "$prog: Can't find $typeTab in any of the following places:\n" .
	join ("\n", @INC) . "\n";
}

#
# Load the type mapping table into `%types', with a crossreference
# from `%keys', and listed in order in `@keys'
#

open (TYPETAB, $typeTab)
    || die "$prog: opening $typeTab: $!\n";
while (<TYPETAB>) {
    next if (/^\s*(#|$)/);
    chop;
    ($nick, $type, $default, $key) = split (/\|/, $_, 4);
    $nick =~ tr/A-Z/a-z/;
    $types{$nick} = {nick=>$nick, type=>$type, default=>$default,
		     key=>$key};
    if ($key ne "") {
	$key =~ tr/A-Z/a-z/;
	push (@keys, $key);
	$keys{$key} = $nick;
    }
}
close (TYPETAB)
    || die "$prog: closing $typeTab: $!\n";
	     print "";		# XXX temporary patch for a coredump bug
				# in Perl.  remove when fixed
#
# If the inputType is not specified, search the input file for keywords
#
# XXX OOPS! this doesn't work on STDIN!
if (!defined ($inputType)) {
    $ii = 0;
    open (TYPETEST, $document)
	|| die "$prog: opening $document: $!\n";
  OUTER: while (($ii++ < 100) && ($_ = <TYPETEST>)) {
        foreach $key (@keys) {
	    if (/$key/i) {
		$inputType = $keys{$key};
		last OUTER;
	    }
	}
    }
    close (TYPETEST)
	|| die "$prog: closing $document: $!\n";
    die "$prog: $document: unknown document type\n"
	if (!defined ($inputType));
}

if (defined ($types{$inputType})) {
    # catch default output type before losing `nick'
    if (!defined ($outputType)) {
	$outputType = $types{$inputType}->{default};
    }
    $inputType = $types{$inputType}->{type};
}

if (defined ($outputFile)) {
    $outputFileString = ">$outputFile";
}

if (defined ($types{$outputType})) {
    $outputType = $types{$outputType}->{type};
}

if ($outputType eq "norm") {
    exec ("$SGMLNORM $document $outputFileString");
} elsif ($outputType eq "isis") {
    exec ("$SGMLS $document $outputFileString");
} else {
    exec ("$SGMLS -l $document | $LIBEXECDIR/defrag -T$outputType -I$inputType $outputFileString");
}

# --- Emacs stuff ---
# Local Variables:
# mode:perl
# End:
